@@ -1,10 +1,41 @@
-# Copyright (c) 2008-2014 Simplistix Ltd
-# See license.txt for license details.
+from textwrap import dedent
 
 from testfixtures import Comparison as C, ShouldRaise, should_raise
 from unittest import TestCase
 
-from .compat import py_33_plus
+from ..compat import PY3, PY_36_PLUS
+from ..shouldraise import ShouldAssert
+
+
+class TestShouldAssert(object):
+
+    def test_no_exception(self):
+        try:
+            with ShouldAssert('foo'):
+                pass
+        except AssertionError as e:
+            assert str(e) == "Expected AssertionError('foo'), None raised!"
+
+    def test_wrong_exception(self):
+        try:
+            with ShouldAssert('foo'):
+                raise KeyError()
+        except KeyError:
+            pass
+
+    def test_wrong_text(self):
+        try:
+            with ShouldAssert('foo'):
+                assert False, 'bar'
+        except AssertionError as e:
+            assert str(e) == dedent("""\
+                --- expected
+                +++ actual
+                @@ -1 +1,2 @@
+                -foo
+                +bar
+                +assert False""")
+
 
 
 class TestShouldRaise(TestCase):
@@ -17,50 +48,42 @@ class TestShouldRaise(TestCase):
     def test_no_exception(self):
         def to_test():
             pass
-        try:
+        with ShouldAssert('ValueError() (expected) != None (raised)'):
             should_raise(ValueError())(to_test)()
-        except AssertionError as e:
-            self.assertEqual(
-                e,
-                C(AssertionError('None raised, ValueError() expected'))
-                )
-        else:
-            self.fail('No exception raised!')
 
     def test_wrong_exception(self):
         def to_test():
             raise ValueError('bar')
-        try:
+        with ShouldAssert(
+            "ValueError('foo',) (expected) != ValueError('bar',) (raised)"
+        ):
             should_raise(ValueError('foo'))(to_test)()
-        except AssertionError as e:
-            self.assertEqual(
-                e,
-                C(AssertionError(
-                    "ValueError('bar',) raised, ValueError('foo',) expected"
-                )))
-        else:
-            self.fail('No exception raised!')
 
     def test_only_exception_class(self):
         def to_test():
             raise ValueError('bar')
         should_raise(ValueError)(to_test)()
 
+    def test_wrong_exception_class(self):
+        def to_test():
+            raise ValueError('bar')
+        if PY3:
+            message = ("<class 'KeyError'> (expected) != "
+                       "ValueError('bar',) (raised)")
+        else:
+            message = ("<type 'exceptions.KeyError'> (expected) != "
+                       "ValueError('bar',) (raised)")
+        with ShouldAssert(message):
+            should_raise(KeyError)(to_test)()
+
     def test_no_supplied_or_raised(self):
         # effectvely we're saying "something should be raised!"
         # but we want to inspect s.raised rather than making
         # an up-front assertion
         def to_test():
             pass
-        try:
+        with ShouldAssert("No exception raised!"):
             should_raise()(to_test)()
-        except AssertionError as e:
-            self.assertEqual(
-                e,
-                C(AssertionError("No exception raised!"))
-                )
-        else:
-            self.fail('No exception raised!')
 
     def test_args(self):
         def to_test(*args):
@@ -141,57 +164,41 @@ class TestShouldRaise(TestCase):
             raise ValueError('foo bar')
 
     def test_with_exception_supplied_wrong_args(self):
-        try:
+        with ShouldAssert(
+            "ValueError('foo',) (expected) != ValueError('bar',) (raised)"
+        ):
             with ShouldRaise(ValueError('foo')):
                 raise ValueError('bar')
-        except AssertionError as e:
-            self.assertEqual(
-                e,
-                C(AssertionError(
-                    "ValueError('bar',) raised, ValueError('foo',) expected"
-                )))
-        else:
-            self.fail('No exception raised!')
 
     def test_neither_supplied(self):
         with ShouldRaise():
             raise ValueError('foo bar')
 
     def test_with_no_exception_when_expected(self):
-        try:
+        with ShouldAssert("ValueError('foo',) (expected) != None (raised)"):
             with ShouldRaise(ValueError('foo')):
                 pass
-        except AssertionError as e:
-            self.assertEqual(
-                e,
-                C(AssertionError("None raised, ValueError('foo',) expected"))
-                )
-        else:
-            self.fail('No exception raised!')
 
     def test_with_no_exception_when_neither_expected(self):
-        try:
+        with ShouldAssert("No exception raised!"):
             with ShouldRaise():
                 pass
-        except AssertionError as e:
-            self.assertEqual(
-                e,
-                C(AssertionError("No exception raised!"))
-                )
-        else:
-            self.fail('No exception raised!')
 
     def test_with_getting_raised_exception(self):
+        e = ValueError('foo bar')
         with ShouldRaise() as s:
-            raise ValueError('foo bar')
-        self.assertEqual(C(ValueError('foo bar')), s.raised)
+            raise e
+        assert e is s.raised
 
     def test_import_errors_1(self):
-        if py_33_plus:
+        if PY3:
             message = "No module named 'textfixtures'"
         else:
             message = 'No module named textfixtures.foo.bar'
-        with ShouldRaise(ImportError(message)):
+
+        exception = ModuleNotFoundError if PY_36_PLUS else ImportError
+
+        with ShouldRaise(exception(message)):
             import textfixtures.foo.bar
 
     def test_import_errors_2(self):
@@ -208,22 +215,17 @@ class TestShouldRaise(TestCase):
             raise FileTypeError('X')
 
     def test_assert_keyerror_raised(self):
-        expected = "KeyError('foo',) raised, AttributeError('foo',) expected"
 
         class Dodgy(dict):
             def __getattr__(self, name):
                 # NB: we forgot to turn our KeyError into an attribute error
                 return self[name]
-        try:
+
+        with ShouldAssert(
+            "AttributeError('foo',) (expected) != KeyError('foo',) (raised)"
+        ):
             with ShouldRaise(AttributeError('foo')):
                 Dodgy().foo
-        except AssertionError as e:
-            self.assertEqual(
-                C(AssertionError(expected)),
-                e
-                )
-        else:
-            self.fail('No exception raised!')
 
     def test_decorator_usage(self):
 
@@ -238,28 +240,20 @@ class TestShouldRaise(TestCase):
             raise AttributeError()
 
     def test_unless_false_bad(self):
-        try:
+        with ShouldAssert("No exception raised!"):
             with ShouldRaise(unless=False):
                 pass
-        except AssertionError as e:
-            self.assertEqual(e, C(AssertionError("No exception raised!")))
-        else:
-            self.fail('No exception raised!')
 
     def test_unless_true_okay(self):
         with ShouldRaise(unless=True):
             pass
 
     def test_unless_true_not_okay(self):
-        try:
+        with ShouldAssert(
+            "AttributeError('foo',) raised, no exception expected"
+        ):
             with ShouldRaise(unless=True):
                 raise AttributeError('foo')
-        except AssertionError as e:
-            self.assertEqual(e, C(AssertionError(
-                "AttributeError('foo',) raised, no exception expected"
-                )))
-        else:
-            self.fail('No exception raised!')
 
     def test_unless_decorator_usage(self):
 
@@ -270,23 +264,17 @@ class TestShouldRaise(TestCase):
         to_test()
 
     def test_identical_reprs(self):
-
         class AnnoyingException(Exception):
             def __init__(self, **kw):
                 self.other = kw.get('other')
 
-        try:
+        with ShouldAssert(
+            "AnnoyingException not as expected:\n\n"
+            'attributes same:\n'
+            "['args']\n\n"
+            "attributes differ:\n"
+            "'other': 'bar' (expected) != 'baz' (raised)\n\n"
+            "While comparing .other: 'bar' (expected) != 'baz' (raised)"
+        ):
             with ShouldRaise(AnnoyingException(other='bar')):
                 raise AnnoyingException(other='baz')
-        except AssertionError as e:
-            print(repr(e))
-            self.assertEqual(
-                C(AssertionError(
-                    "AnnoyingException() raised, AnnoyingException() expected,"
-                    " attributes differ:\n"
-                    "  other:'bar' != 'baz'"
-                )),
-                e,
-                )
-        else:
-            self.fail('No exception raised!')
